"CMFDemo - organize and create CMF demo sites."

import CMFTour
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import DTMLFile
from Acquisition import Implicit
from Globals import Persistent, MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from OFS.DTMLMethod import DTMLMethod
from OFS.ObjectManager import BadRequestException
from OFS.Image import Image
from Products.BTreeFolder.BTreeFolder import BTreeFolder

from Products.CMFDefault.Portal import manage_addCMFSite
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.NewsItem import addNewsItem

# type install imports
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFDemoNewsItem import DemoNewsItem, demoNewsItem_globals
import string

import UpdateCMFTopic

from urllib import quote

# content strings, DTML, etc.

productDir = '%s/lib/python/Products/CMFDemo/' % INSTANCE_HOME
customDir = productDir + 'custom/'
tourDir = productDir + 'tour_files/'
tourId = 'CMFTour'
portalDescriptionStr = open(customDir + 'portalDescription.stx').read()

login_formStr = open(customDir + 'login_form.dtml').read()
logged_inStr = open(customDir + 'logged_in.dtml').read()
first_logged_inStr = open(customDir + 'first_logged_in.dtml').read()
news_boxStr = open(customDir + 'news_box.dtml').read()
recent_newsStr = open(customDir + 'recent_news.dtml').read()
standard_top_barStr = open(customDir + 'standard_top_bar.dtml').read()
logo = open(customDir + 'zbc.gif').read()

def getPassword(portal, name):
    "Return the member password for the named member."
    membership = getToolByName(portal, 'portal_membership')
    memberName = membership.getAuthenticatedMember().name
    return portal.acl_users.getUser(name).__

def getLoginURL(portal, name):
    "Return a relative URL to a logged-in page for the named member."
    password = getPassword(portal, name)
    return "logged_in?__ac_name=" + name + "&__ac_password=" + quote(password)

class CMFDemoActionProvider:
    "A class to provide actions for the CMF Demo."

    def listActions(self, info):
        "Return a list of actions for the actions box."

        portal_url = info.portal_url
        tour_url = info.portal.tour_URL

        membership = getToolByName(info.portal, 'portal_membership')
        memberName = membership.getAuthenticatedMember().name

        switchArg = '&autoLogin=true'
        m_url = getLoginURL(info.portal, "Writer") + switchArg
        r_url = getLoginURL(info.portal, "Editor") + switchArg

        actions = [{'name': 'Tour',
                    'url': tour_url,
                    'permissions': [],
                    'category': 'global'},]

        if memberName == "Writer":
            # add links to Writer, Editor login
            # this means that any non-anon user can get these passwords
            actions.extend([{'name': 'Switch to Editor',
                             'url': r_url,
                             'permissions' : [],
                             'category': 'global'},])
        elif memberName == "Editor":
            actions.extend([{'name': 'Switch to Writer',
                             'url': m_url,
                             'permissions' : [],
                             'category': 'global'},])            
            
        return actions

def addCMFDemoGenerator(self, id, title = None, REQUEST=None):
    "Add a CMFDemoGenerator instance."

    CMFTour.loadTour(self, tourDir)

    folder = BTreeFolder()
    folder.id = folder.title = "DemoCMFSites"
    folderId = self._setObject(folder.id, folder)

    ob = CMFDemoGenerator(id, tourId, folderId)
    ob.title = str(title)
    id = self._setObject(id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')

addCMFDemoGeneratorForm = DTMLFile('dtml/addCMFDemoGeneratorForm',
                                   globals())

class CMFDemoGenerator(Implicit, Persistent, RoleManager, PropertyManager,
                       Item):
    "Generator class for CMF Demo sites."

    meta_type='CMFDemoGenerator'

    security = ClassSecurityInfo()

    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/main', globals())

    manage_options=(
        {'label':'Main', 'action':'manage_main'},
        {'label':'Add Site', 'action':'manage_addSite'},
        ) + RoleManager.manage_options + PropertyManager.manage_options

    def __init__(self, id, tourId, siteFolderId):
        self.id = id
        self.tourId = tourId
        self.siteFolderId = siteFolderId

    security.declarePublic('manage_addSite') 
    manage_addSite = DTMLFile('dtml/addSite', globals())

    index_html = manage_addSite

    security.declarePublic('addSite')
    def addSite(self, siteId, password, passwordConfirm, email,
                REQUEST = None):
        "Add a CMF demo site."

        vpOut = self.verifyCMFPassword(password, passwordConfirm)
        if vpOut:
            return vpOut

        siteFolder = getattr(self.aq_parent, self.siteFolderId)

        try:
            manage_addCMFSite(siteFolder, siteId,
                              title = "Demo Portal",
                              description = portalDescriptionStr)
        except BadRequestException:
            return self.manage_addSite(error=1)
        
        portal = getattr(siteFolder, siteId)

        self.installTopics(portal)
        self.installDemoNewsItem(portal)
        self.customizeProperties(portal)
        self.customizeSkins(portal)
        #self.customizeExistingMember(portal, password, REQUEST)
        self.customizeMembers(portal, password, REQUEST)
        self.customizeActions(portal)
        # this isn't needed when we use the demo news item type
        #self.customizeMetadata(portal)

        if REQUEST is not None:
            # redirect to CMF site
            u = REQUEST['URL2'] 
            REQUEST.RESPONSE.redirect(portal.absolute_url() + '/' +
                                      getLoginURL(portal, "Writer") +
                                      '&came_from=first_logged_in')

    def installTopics(self, portal):
        "Setup this portal to use topics.  Hack hack kluge."
        UpdateCMFTopic.update(portal)
        portal.aq_parent.manage_permission('Add portal topics',
                                           ['Anonymous', 'Manager', 'Owner'],
                                           acquire='on')

    def customizeProperties(self, portal):
        "Add properties to the portal that the demo will use."
        tourURL = "javascript:window.open(" + \
                  "'CMFTour/creating_content'," + \
                  "'CMFTour'," + \
                  "'width=376," + \
                  "height=341," + \
                  "left=0," + \
                  "top=0," + \
                  "screenX=0," + \
                  "screenY=0," + \
                  "menubar=no," + \
                  "toolbar=no," + \
                  "scrollbars=yes," + \
                  "resizable=yes').focus();"
                  
        portal.manage_addProperty('tour_URL', tourURL, 'string')
        
    def customizeMembers(self, portal, password, REQUEST):
        "Add members for the portal demo."
        # we want to replicate CMFDefault/skins/control/register.dtml
        # this is called by CMFDefault/skins/generic/join_form,
        # and 'username', 'email' are expected to be stuffed into REQUEST
        # and used as the properties arg.
        # Another way might be to override these skins, but we want users
        # to be able to add members in the normal way, too.
        portal_registration = getToolByName(portal, "portal_registration")
        REQUEST['listed'] = 'on'
        
        REQUEST['username'] = 'Writer'
        portal_registration.addMember('Writer', password,
                                      properties = REQUEST)

        REQUEST['username'] = 'Editor'
        # can't add reviewer on creation - hardcoded roles!
        #portal.portal_registration.addMember('Editor', email,
        #                                     ('Member', 'Editor'),
        #                                     properties = REQUEST)
        portal_registration.addMember('Editor', password,
                                      properties = REQUEST)
        portal.acl_users._changeUser('Editor', None, None,
                                     ['Member', 'Reviewer'], None)
        

    def customizeExistingMember(self, portal, password, REQUEST):
        "Add an existing member and content to the portal."
        #req = REQUEST.clone()
        portal_registration = getToolByName(portal, "portal_registration")
        REQUEST['listed'] = 'on'        
        REQUEST['username'] = 'ExistingWriter'
        portal_registration.addMember('ExistingWriter', password,
                                      properties = REQUEST)

        pm = getToolByName(portal, 'portal_membership')
        exHome = pm.getHomeFolder('ExistingWriter')
        from itemFields import *
        #exHome.addNewsItem(presElect1Id, presElect1Title, presElect1Desc,
        #                      presElect1Text)
        addNewsItem(exHome, presElect1Id)
        #can't do this because security gets called
        #exHome.invokeFactory('News Item', presElect1Id)
        p1Obj = getattr(exHome, presElect1Id)
        #can't do this because it's a pythonscript that calls
        #restrictedTraverse
        #presElect1Obj.metadata_edit(title = presElect1Title,
        #                            description = presElect1Desc,
        #                            subject = presElect1Subject,
        #                            #contributors = None,
        #                            #effective_date = None,
        #                            #expiration_date = None,
        #                            #format = None,
        #                            #language = None,
        #                            #rights = None
        #                            )
        # this skips actions in metadata_edit.py
        p1Obj.editMetadata(title = presElect1Title,
                           description = presElect1Desc,
                           subject = presElect1Subject,
                           contributors = p1Obj.Contributors(),
                           effective_date = p1Obj.EffectiveDate(),
                           expiration_date = p1Obj.expires(),
                           format = p1Obj.Format(),
                           language = p1Obj.Language(),
                           rights = p1Obj.Rights())
        p1Obj.edit(text = presElect1Text, description = presElect1Desc)
        # prepare to submit the item
        pw = getToolByName(portal, 'portal_workflow')
        # kluge: set perms on the member folder to allow anon to submit - 
        # since we won't have a security context we'll be anon
        # the right way to do this would be to use a different
        # workflow than default_workflow that doesn't check
        exHome.manage_permission('Request review', ['Anonymous', 'Member'],
                                 acquire='on')
        # actually submit the item
        pw.doActionFor(p1Obj, 'submit', comment = None)
        
    def customizeMetadata(self, portal):
        "Pre-set the metadata for the demo."
        portal_metadata = getToolByName(portal, "portal_metadata")
        portal_metadata.updateElementPolicy('Subject', 'News Item', 0, 1, None,
                                            0, ['Politics', 'Sports',
                                                'Economics'])
        
    def customizeSkins(self, portal):
        "copy our custom skins to the new CMF site"
        # we need to create these methods here, not in the module,
        # else we get a foreign db connection exception
        logged_in = DTMLMethod(logged_inStr, __name__ = "logged_in")
        login_form = DTMLMethod(login_formStr, __name__ = "login_form")
        first_logged_in = DTMLMethod(first_logged_inStr,
                                     __name__ = "first_logged_in")
        news_box = DTMLMethod(news_boxStr, __name__ = "news_box")
        recent_news = DTMLMethod(recent_newsStr, __name__ = "recent_news")

        standard_top_bar = DTMLMethod(standard_top_barStr,
                                     __name__ = "standard_top_bar")
        zbc_gif = Image(id='zbc.gif', title='zbc.gif', file=logo)
       
        ps = getToolByName(portal, "portal_skins")
        ps.manage_addProduct['OFSP'].manage_addFolder(id='tour')
        basicSel = ps._getSelections()['Basic']
        tourSel = 'tour, ' + basicSel
        ps.addSkinSelection('Tour', tourSel, make_default = 1)

        skinFolder = getattr(ps, 'tour')
        skinFolder._setObject('login_form', login_form)
        skinFolder._setObject('logged_in', logged_in)
        skinFolder._setObject('first_logged_in', first_logged_in)
        skinFolder._setObject('news_box', news_box)
        skinFolder._setObject('recent_news', recent_news)
        skinFolder._setObject('standard_top_bar', standard_top_bar)
        skinFolder._setObject('zbc.gif', zbc_gif)

    def customizeActions(self, portal):
        "Add demo actions to the portal actions tool."
        # we could have only one provider for all portals
        portal.demo_actions_provider = CMFDemoActionProvider()
        portal_actions = getToolByName(portal, "portal_actions")
        portal_actions.action_providers = (portal_actions.action_providers +
                                           ('demo_actions_provider',))

    def installDemoNewsItem(self, portal):
        "Install the DemoNewsItem type."
        typestool = getToolByName(portal, 'portal_types')
        skinstool = getToolByName(portal, 'portal_skins')
        metadatatool = getToolByName(portal, 'portal_metadata')

        # Borrowed from CMFDefault.Portal.PortalGenerator.setupTypes()
        # We loop through anything defined in the factory type information
        # and configure it in the types tool if it doesn't already exist
        for t in DemoNewsItem.factory_type_information:
            if t['id'] not in typestool.objectIds():
                cfm = apply(ContentFactoryMetadata, (), t)
                typestool._setObject(t['id'], cfm)

        # Setup an MetadataTool Element Policy for demoNewsItems

        #the default stuff doesn't seem to do it.
        #so use a skin to supply the default.
        metadatatool.addElementPolicy(element='Subject'
                                      ,content_type='Demo News Item'
                                      ,is_required=0
                                      ,supply_default=0
                                      ,default_value=''
                                      ,enforce_vocabulary=0
                                      ,allowed_vocabulary=('Politics',
                                                           'Economics',
                                                           'Sports')
                                      ,REQUEST=None
                                      )

        # Setup the skins
        # This is borrowed from CMFDefault/scripts/addImagesToSkinPaths.pys
        if 'demoNewsItem' not in skinstool.objectIds():
            # We need to add Filesystem Directory Views for any directories
            # in our skins/ directory.  These directories should already be
            # configured.
            addDirectoryViews(skinstool, 'skins', demoNewsItem_globals)

        # Now we need to go through the skin configurations and insert
        # 'demoNewsItem' into the configurations.  Preferably, this should be
        # right before where 'content' is placed.  Otherwise, we append
        # it to the end.
        skins = skinstool.getSkinSelections()
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if 'demoNewsItem' not in path:
                try: path.insert(path.index('content'), 'demoNewsItem')
                except ValueError:
                    path.append('demoNewsItem')
                
                path = string.join(path, ', ')
                # addSkinSelection will replace exissting skins as well.
                skinstool.addSkinSelection(skin, path)
        
    def verifyCMFPassword(self, password, passwordConfirm):
        "Verify the password how the CMF will do."
        # perform these checks from RegistrationTool now
        # to avoid CMF error pages
        if len(password) < 5:
            return MessageDialog(
                title= 'Password Error',
                message = 'Your password must contain at least 5 characters.'\
                ' Please try again.',
                action='manage_addSite')
        if passwordConfirm != password:
            return MessageDialog(
                title= 'Password Error',
                message = 'Your password and confirmation did not match.' \
                ' Please try again.',
                action='manage_addSite')
        return None

InitializeClass(CMFDemoGenerator)
