import os, sys
from OFS.DTMLDocument import DTMLDocument
from OFS.DTMLMethod import DTMLMethod
from OFS.Folder import Folder

stepDocumentStr = """
<dtml-var standard_html_header>
<dtml-var body fmt="structured-text">
<dtml-var step_html_footer>
                  """
noteDocumentStr = """
<dtml-var standard_html_header>
<dtml-var body fmt="structured-text">
<dtml-var note_html_footer>
                  """

tour_text = ('step_files', 'note_files')

class TourFolder(Folder):
    """
    A simple extension of OFS.Folder providing next and previous methods
    """
 
    def next(self, id):
        nextindex=self.order.index(id) + 1        
        
        # list index may be out of range if pos is the last item.
        try:
            next = self.order[nextindex]
        except IndexError:
            return None    
        nextObj = getattr(self, next)
        nextURL = nextObj.getId() #s/b returning absolute_url here,
                                  #but this won't work for some reason
        return nextURL 

    def previous(self, id):
              
        previousindex = self.order.index(id) - 1
        if previousindex == -1:
            previousURL = None
        else:
            previous = self.order[previousindex]
            previousObj = getattr(self, previous)
            previousURL = previousObj.getId()
        return previousURL

def loadTour(root, dir):
    """
    create a Folder for the tour application; load header and footer methods
    from fs; call loadSteps to iterate through
    remaining items to instantiate tour documents from fs
    """
    sys.path.append(dir)
    modname = 'tourPropertyMap'
    propMap = __import__(modname) 
    
    tourRoot = TourFolder(__name__='CMFTour')
    tourFolderId = root._setObject('CMFTour', tourRoot)
    tourFolder = root._getOb(tourFolderId) 
    tourFolder.id='CMFTour'
    tourFolder.title='CMF Tour'
    tourFolder.order=[]
    for dict in propMap.step_files:
        tourFolder.order.append(dict['id'])

    for dict in propMap.common_files:
        commonStr = open(dir + dict['id'] + '.dtml').read()
        commonfile = DTMLMethod(commonStr)
        tourFolder._setObject(dict['id'], commonfile)
    
    initialize_tour_files(dir, tourFolder, propMap, ('step', 'note'))
    
def initialize_tour_files(dir, folder, propertyMap, tourComponents):

    for attr in tourComponents:
        for dict in getattr(propertyMap, attr + '_files'):
            fileStr = open(dir + dict['id'] + '.stx').read()
            fileObj = DTMLDocument(eval(attr + 'DocumentStr'))
            fileId = folder._setObject(dict['id'], fileObj)
            file = folder._getOb(fileId)
            for property in dict.keys():
                 setattr(file, property, dict[property])
            file.body = fileStr     
 


